/* PCF2129AT Arduino用ライブラリ ver 0.5
 * memo: 11/5 22.30
 * 11/16 00:00
 *
 * NXPより公開されているmbed向けlibraryおよび
 * CQ出版より公開されているサンプルコードを
 * 参考にしています.
*/

#include <Arduino.h>
#include <Wire.h>
#include "PCF2129AT.h"

int cmd[8];//送受信用のキャッシュ


/*初期化*/
void PCF2129AT::init()
{
  //デバイスの仕様としては
  //24時間モードか12時間モードの設定が出来たり
  //アラームの設定が出来たり
  //外部に割り込みの出力が出来たり
  //etc....

}

/*日時の取得*/
void  PCF2129AT::getTime(dt_dat *dt)
{
  PCF2129AT::dataRead(PCF2129AT_ADDR,  Seconds, cmd, 7);
  cmd[0] &= 0x7f;                         // 有効なのは下位7ビット
  dt->s = (cmd[0] >> 4) * 10 + (cmd[0] & 0xf);    // BCDの数値化

  cmd[1] &= 0x7f;                         // 有効なのは下位7ビット
  dt->m = (cmd[1] >> 4) * 10 + (cmd[1] & 0xf);    // BCDの数値化

  cmd[2] &= 0x3f;                         // 有効なのは下位6ビット
  dt->h = (cmd[2] >> 4) * 10 + (cmd[2] & 0xf);    // BCDの数値化

  cmd[3] &= 0x3f;                         // 有効なのは下位6ビット
  dt->d = (cmd[3] >> 4) * 10 + (cmd[3] & 0xf);    // BCDの数値化

  dt->wd = cmd[4];                        // BCDの数値化

  cmd[5] &= 0x1f;                         // 有効なのは下位5ビット
  dt->mm = (cmd[5] >> 4) * 10 + (cmd[5] & 0xf);   // BCDの数値化

  dt->y = (cmd[6] >> 4) * 10 + (cmd[6] & 0xf);    // BCDの数値化

}


/*日時の設定*/
void PCF2129AT::setDate(dt_dat *dt)
{
    cmd[0] = ((dt->s / 10) << 4) + (dt->s % 10) + 0x80; // 秒のBCD化
    cmd[1] = ((dt->m / 10) << 4) + (dt->m % 10);        // 分のBCD化
    cmd[2] = ((dt->h / 10) << 4) + (dt->h % 10);        // 時のBCD化
    cmd[3] = ((dt->d / 10) << 4) + (dt->d % 10);        // 日のBCD化
    cmd[4] = dt->wd;
    cmd[5] = ((dt->mm / 10) << 4) + (dt->mm % 10);      // 月のBCD化
    dt->y = dt->y - 2000;
    cmd[6] = ((dt->y / 10) << 4) + (dt->y % 10);        // 年のBCD化
    PCF2129AT::dataSend(PCF2129AT_ADDR,  Seconds, cmd, 7);      // 日時の設定
}



//I2C通信で複数受信
//https://code.google.com/p/arduino/issues/detail?id=28
/*I2Cで配列のデータを送る*/
/*送信先アドレス,送信先レジスタ,送信する配列データ,送信する配列要素数*/
void PCF2129AT::dataSend(int id,int reg,int *data,int datasize)
{
  Wire.beginTransmission(id);
  Wire.write(reg);
  for(int i=0;i<datasize;i++)
  {
    Wire.write(data[i]);
    //Serial.print("data sended : "); //debug
    //Serial.println(data[i], BIN); //debug
  }
  Wire.endTransmission();
}

/*I2Cで配列のデータを受け取る*/
/*受信元アドレス,受信元レジスタ,受信する配列データ,受信する配列要素数*/
void PCF2129AT::dataRead(int id,int reg,int *data,int datasize)
{
  //Serial.print("transmission begin");//debug
  Wire.beginTransmission(id);             //スレーブアドレス送信,書き込みモード
  Wire.write(reg);                        //レジスタアドレス送信
  Wire.endTransmission(true);            //STOPコンディションを送る
  //STOPコンディションを送る仕様
  Wire.requestFrom(id, datasize, false);  //スレーブアドレス送信,受信モード,
  for(int i=0;i<datasize;i++)
  {
    data[i] = Wire.read();
    //Serial.print("data recived : ");//debug
    //Serial.println(data[i], BIN);//debug
  }
  Wire.endTransmission(true);
  //Serial.println("transmission end");//debug
}
